#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.greece import Greece
from tests.common import CommonCountryTests


class TestGreece(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Greece)

    def test_new_years_day(self):
        self.assertHolidayName("Πρωτοχρονιά", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName("Θεοφάνεια", (f"{year}-01-06" for year in self.full_range))

    def test_clean_monday(self):
        name = "Καθαρά Δευτέρα"
        self.assertHolidayName(
            name,
            "2020-03-02",
            "2021-03-15",
            "2022-03-07",
            "2023-02-27",
            "2024-03-18",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_independence_day(self):
        self.assertHolidayName(
            "Εικοστή Πέμπτη Μαρτίου", (f"{year}-03-25" for year in self.full_range)
        )

    def test_good_friday(self):
        name = "Μεγάλη Παρασκευή"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Δευτέρα του Πάσχα"
        self.assertHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Εργατική Πρωτομαγιά"
        self.assertHolidayName(
            name,
            "2024-05-07",
            (
                f"{year}-05-01"
                for year in (*range(self.start_year, 2024), *range(2025, self.end_year))
            ),
        )
        obs_dt = (
            "2021-05-04",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (παρατηρήθηκε)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_whit_monday(self):
        name = "Δευτέρα του Αγίου Πνεύματος"
        self.assertHolidayName(
            name,
            "2020-06-08",
            "2021-06-21",
            "2022-06-13",
            "2023-06-05",
            "2024-06-24",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName(
            "Κοίμηση της Θεοτόκου", (f"{year}-08-15" for year in self.full_range)
        )

    def test_ochi_day(self):
        self.assertHolidayName("Ημέρα του Όχι", (f"{year}-10-28" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Παραμονή Χριστουγέννων"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Χριστούγεννα", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        self.assertHolidayName(
            "Σύναξη της Υπεραγίας Θεοτόκου", (f"{year}-12-26" for year in self.full_range)
        )

    def test_new_years_eve(self):
        name = "Παραμονή Πρωτοχρονιάς"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Πρωτοχρονιά"),
            ("2024-01-06", "Θεοφάνεια"),
            ("2024-03-18", "Καθαρά Δευτέρα"),
            ("2024-03-25", "Εικοστή Πέμπτη Μαρτίου"),
            ("2024-05-03", "Μεγάλη Παρασκευή"),
            ("2024-05-06", "Δευτέρα του Πάσχα"),
            ("2024-05-07", "Εργατική Πρωτομαγιά"),
            ("2024-06-24", "Δευτέρα του Αγίου Πνεύματος"),
            ("2024-08-15", "Κοίμηση της Θεοτόκου"),
            ("2024-10-28", "Ημέρα του Όχι"),
            ("2024-12-25", "Χριστούγεννα"),
            ("2024-12-26", "Σύναξη της Υπεραγίας Θεοτόκου"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Πρωτοχρονιά"),
            ("2022-01-06", "Θεοφάνεια"),
            ("2022-03-07", "Καθαρά Δευτέρα"),
            ("2022-03-25", "Εικοστή Πέμπτη Μαρτίου"),
            ("2022-04-22", "Μεγάλη Παρασκευή"),
            ("2022-04-25", "Δευτέρα του Πάσχα"),
            ("2022-05-01", "Εργατική Πρωτομαγιά"),
            ("2022-05-02", "Εργατική Πρωτομαγιά (παρατηρήθηκε)"),
            ("2022-06-13", "Δευτέρα του Αγίου Πνεύματος"),
            ("2022-08-15", "Κοίμηση της Θεοτόκου"),
            ("2022-10-28", "Ημέρα του Όχι"),
            ("2022-12-24", "Παραμονή Χριστουγέννων"),
            ("2022-12-25", "Χριστούγεννα"),
            ("2022-12-26", "Σύναξη της Υπεραγίας Θεοτόκου"),
            ("2022-12-31", "Παραμονή Πρωτοχρονιάς"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-03-07", "Green Monday"),
            ("2022-03-25", "Independence Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-25", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day (observed)"),
            ("2022-06-13", "Whit Monday"),
            ("2022-08-15", "Dormition of the Mother of God"),
            ("2022-10-28", "Ochi Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Glorifying Mother of God"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-03-07", "Чистий понеділок"),
            ("2022-03-25", "День незалежності"),
            ("2022-04-22", "Страсна пʼятниця"),
            ("2022-04-25", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-02", "День праці (вихідний)"),
            ("2022-06-13", "День Святого Духа"),
            ("2022-08-15", "Успіння Пресвятої Богородиці"),
            ("2022-10-28", "День Охі"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Собор Пресвятої Богородиці"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.japan_exchange import JapanExchange
from tests.common import CommonFinancialTests


class TestJapanExchange(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(JapanExchange)

    def test_code(self):
        self.assertTrue(hasattr(self.holidays, "market"))
        self.assertIsNone(getattr(self.holidays, "country", None))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "元日; 銀行休業日"),
            ("2024-01-02", "銀行休業日"),
            ("2024-01-03", "銀行休業日"),
            ("2024-01-08", "成人の日"),
            ("2024-02-11", "建国記念の日"),
            ("2024-02-12", "振替休日"),
            ("2024-02-23", "天皇誕生日"),
            ("2024-03-20", "春分の日"),
            ("2024-04-29", "昭和の日"),
            ("2024-05-03", "憲法記念日"),
            ("2024-05-04", "みどりの日"),
            ("2024-05-05", "こどもの日"),
            ("2024-05-06", "振替休日"),
            ("2024-07-15", "海の日"),
            ("2024-08-11", "山の日"),
            ("2024-08-12", "振替休日"),
            ("2024-09-16", "敬老の日"),
            ("2024-09-22", "秋分の日"),
            ("2024-09-23", "振替休日"),
            ("2024-10-14", "スポーツの日"),
            ("2024-11-03", "文化の日"),
            ("2024-11-04", "振替休日"),
            ("2024-11-23", "勤労感謝の日"),
            ("2024-12-31", "銀行休業日"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "Bank Holiday; New Year's Day"),
            ("2024-01-02", "Bank Holiday"),
            ("2024-01-03", "Bank Holiday"),
            ("2024-01-08", "Coming of Age Day"),
            ("2024-02-11", "Foundation Day"),
            ("2024-02-12", "Substitute Holiday"),
            ("2024-02-23", "Emperor's Birthday"),
            ("2024-03-20", "Vernal Equinox Day"),
            ("2024-04-29", "Showa Day"),
            ("2024-05-03", "Constitution Day"),
            ("2024-05-04", "Greenery Day"),
            ("2024-05-05", "Children's Day"),
            ("2024-05-06", "Substitute Holiday"),
            ("2024-07-15", "Marine Day"),
            ("2024-08-11", "Mountain Day"),
            ("2024-08-12", "Substitute Holiday"),
            ("2024-09-16", "Respect for the Aged Day"),
            ("2024-09-22", "Autumnal Equinox Day"),
            ("2024-09-23", "Substitute Holiday"),
            ("2024-10-14", "Sports Day"),
            ("2024-11-03", "Culture Day"),
            ("2024-11-04", "Substitute Holiday"),
            ("2024-11-23", "Labor Thanksgiving Day"),
            ("2024-12-31", "Bank Holiday"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่; วันหยุดธนาคาร"),
            ("2024-01-02", "วันหยุดธนาคาร"),
            ("2024-01-03", "วันหยุดธนาคาร"),
            ("2024-01-08", "วันฉลองบรรลุนิติภาวะ"),
            ("2024-02-11", "วันชาติญี่ปุ่น"),
            ("2024-02-12", "วันหยุดชดเชย"),
            ("2024-02-23", "วันคล้ายวันพระราชสมภพ สมเด็จพระจักรพรรดินารุฮิโตะ"),
            ("2024-03-20", "วันวสันตวิษุวัต"),
            ("2024-04-29", "วันโชวะ"),
            ("2024-05-03", "วันรัฐธรรมนูญ"),
            ("2024-05-04", "วันพฤกษชาติ"),
            ("2024-05-05", "วันเด็กแห่งชาติ"),
            ("2024-05-06", "วันหยุดชดเชย"),
            ("2024-07-15", "วันแห่งทะเล"),
            ("2024-08-11", "วันแห่งภูเขา"),
            ("2024-08-12", "วันหยุดชดเชย"),
            ("2024-09-16", "วันเคารพผู้สูงอายุ"),
            ("2024-09-22", "วันศารทวิษุวัต"),
            ("2024-09-23", "วันหยุดชดเชย"),
            ("2024-10-14", "วันกีฬาแห่งชาติ"),
            ("2024-11-03", "วันวัฒนธรรม"),
            ("2024-11-04", "วันหยุดชดเชย"),
            ("2024-11-23", "วันขอบคุณแรงงาน"),
            ("2024-12-31", "วันหยุดธนาคาร"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.samoa import Samoa
from tests.common import CommonCountryTests


class TestSamoa(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Samoa, years=range(1950, 2050))

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1950, 2050)))

    def test_new_years_second_day(self):
        self.assertHolidayName(
            "The Day After New Year's Day", (f"{year}-01-02" for year in range(1950, 2050))
        )

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-06-01" for year in range(1950, 2050)))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1950, 2050)))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in range(1950, 2050)))

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "The Day After New Year's Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Day After Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-15", "Mother's Day"),
            ("2023-06-01", "Independence Day"),
            ("2023-08-14", "Father's Day"),
            ("2023-10-09", "White Sunday (Lotu a Tamaiti)"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "The Day After New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Day After Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-13", "Mother's Day"),
            ("2024-06-01", "Independence Day"),
            ("2024-08-12", "Father's Day"),
            ("2024-10-14", "White Sunday (Lotu a Tamaiti)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.haiti import Haiti
from tests.common import CommonCountryTests


class TestHaiti(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Haiti)

    def test_national_independence_day(self):
        self.assertHolidayName(
            "Fête de l'Indépendance Nationale", (f"{year}-01-01" for year in self.full_range)
        )

    def test_ancestry_day(self):
        self.assertHolidayName("Jour des Aïeux", (f"{year}-01-02" for year in self.full_range))

    def test_agriculture_and_labor_day(self):
        self.assertHolidayName(
            "Fête de l'Agriculture et du Travail", (f"{year}-05-01" for year in self.full_range)
        )

    def test_flag_day_and_university_day(self):
        self.assertHolidayName(
            "Fête du Drapeau et de l'Université", (f"{year}-05-18" for year in self.full_range)
        )

    def test_commemoration_of_the_battle_of_vertieres(self):
        self.assertHolidayName(
            "Commémoration de la Bataille de Vertières",
            (f"{year}-11-18" for year in self.full_range),
        )

    def test_armed_forces_day(self):
        self.assertHolidayName(
            "Jour des Forces Armées", (f"{year}-11-18" for year in self.full_range)
        )

    def test_new_years_day(self):
        self.assertHolidayName("Nouvel An", (f"{year}-01-01" for year in self.full_range))

    def test_shrove_monday(self):
        name = "Lundi Gras"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_fat_tuesday(self):
        name = "Mardi Gras"
        self.assertHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Vendredi Saint"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Pâques"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Fête-Dieu"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName(
            "Assomption de Marie", (f"{year}-08-15" for year in self.full_range)
        )

    def test_death_of_dessalines(self):
        self.assertHolidayName("Mort de Dessalines", (f"{year}-10-17" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("La Toussaint", (f"{year}-11-01" for year in self.full_range))

    def test_day_of_the_dead(self):
        self.assertHolidayName("Fête des Morts", (f"{year}-11-02" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in self.full_range))

    def test_ash_wednesday(self):
        name = "Mercredi des Cendres"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
            "2025-03-05",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_maundy_thursday(self):
        name = "Jeudi Saint"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_national_sovereignty_day(self):
        name = "Fête de la Souveraineté Nationale"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-05-23" for year in self.full_range))

    def test_feast_of_lady_of_perpetual_help_patroness_of_haiti(self):
        name = "Fête de Notre-Dame du Perpétuel Secours, patronne d'Haiti"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-06-27" for year in self.full_range))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_birth_anniversary_of_jean_jacques_dessalines(self):
        name = "Anniversaire de Naissance de Jean-Jacques Dessalines"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-09-20" for year in self.full_range))

    def test_discovery_day(self):
        name = "Jour de la Découverte"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-05" for year in self.full_range))

    def test_2023_public_holiday(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Fête de l'Indépendance Nationale; Nouvel An"),
            ("2023-01-02", "Jour des Aïeux"),
            ("2023-02-19", "Carnaval"),
            ("2023-02-20", "Lundi Gras"),
            ("2023-02-21", "Mardi Gras"),
            ("2023-04-07", "Vendredi Saint"),
            ("2023-04-09", "Pâques"),
            ("2023-05-01", "Fête de l'Agriculture et du Travail"),
            ("2023-05-18", "Fête du Drapeau et de l'Université"),
            ("2023-06-08", "Fête-Dieu"),
            ("2023-08-15", "Assomption de Marie"),
            ("2023-10-17", "Mort de Dessalines"),
            ("2023-11-01", "La Toussaint"),
            ("2023-11-02", "Fête des Morts"),
            ("2023-11-18", "Commémoration de la Bataille de Vertières; Jour des Forces Armées"),
            ("2023-12-25", "Noël"),
        )

    def test_2025_public_holiday(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "Fête de l'Indépendance Nationale; Nouvel An"),
            ("2025-01-02", "Jour des Aïeux"),
            ("2025-03-02", "Carnaval"),
            ("2025-03-03", "Lundi Gras"),
            ("2025-03-04", "Mardi Gras"),
            ("2025-04-18", "Vendredi Saint"),
            ("2025-04-20", "Pâques"),
            ("2025-05-01", "Fête de l'Agriculture et du Travail"),
            ("2025-05-18", "Fête du Drapeau et de l'Université"),
            ("2025-06-19", "Fête-Dieu"),
            ("2025-08-15", "Assomption de Marie"),
            ("2025-10-17", "Mort de Dessalines"),
            ("2025-11-01", "La Toussaint"),
            ("2025-11-02", "Fête des Morts"),
            ("2025-11-18", "Commémoration de la Bataille de Vertières; Jour des Forces Armées"),
            ("2025-12-25", "Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Fête de l'Indépendance Nationale; Nouvel An"),
            ("2024-01-02", "Jour des Aïeux"),
            ("2024-02-11", "Carnaval"),
            ("2024-02-12", "Lundi Gras"),
            ("2024-02-13", "Mardi Gras"),
            ("2024-02-14", "Mercredi des Cendres"),
            ("2024-03-28", "Jeudi Saint"),
            ("2024-03-29", "Vendredi Saint"),
            ("2024-03-31", "Pâques"),
            ("2024-05-01", "Fête de l'Agriculture et du Travail"),
            ("2024-05-09", "Ascension"),
            ("2024-05-18", "Fête du Drapeau et de l'Université"),
            ("2024-05-23", "Fête de la Souveraineté Nationale"),
            ("2024-05-30", "Fête-Dieu"),
            ("2024-06-27", "Fête de Notre-Dame du Perpétuel Secours, patronne d'Haiti"),
            ("2024-08-15", "Assomption de Marie"),
            ("2024-09-20", "Anniversaire de Naissance de Jean-Jacques Dessalines"),
            ("2024-10-17", "Mort de Dessalines"),
            ("2024-11-01", "La Toussaint"),
            ("2024-11-02", "Fête des Morts"),
            ("2024-11-18", "Commémoration de la Bataille de Vertières; Jour des Forces Armées"),
            ("2024-12-05", "Jour de la Découverte"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "National Independence Day; New Year's Day"),
            ("2024-01-02", "Ancestry Day"),
            ("2024-02-11", "Carnival"),
            ("2024-02-12", "Shrove Monday"),
            ("2024-02-13", "Fat Tuesday"),
            ("2024-02-14", "Ash Wednesday"),
            ("2024-03-28", "Maundy Thursday"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-31", "Easter Sunday"),
            ("2024-05-01", "Agriculture and Labor Day"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-18", "Flag Day and University Day"),
            ("2024-05-23", "National Sovereignty Day"),
            ("2024-05-30", "Corpus Christi"),
            ("2024-06-27", "Feast of Lady of Perpetual Help, Patroness of Haiti"),
            ("2024-08-15", "Assumption Day"),
            ("2024-09-20", "Birth Anniversary of Jean-Jacques Dessalines"),
            ("2024-10-17", "Death of Dessalines"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-02", "Day of the Dead"),
            ("2024-11-18", "Armed Forces Day; Commemoration of the Battle of Vertieres"),
            ("2024-12-05", "Discovery Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_es(self):
        self.assertLocalizedHolidays(
            "es",
            ("2024-01-01", "Año Nuevo; Día de la Independencia Nacional"),
            ("2024-01-02", "Fiesta de los Antepasados"),
            ("2024-02-11", "Carnaval"),
            ("2024-02-12", "Lunes de Carnaval"),
            ("2024-02-13", "Martes Gordo"),
            ("2024-02-14", "Miércoles de Ceniza"),
            ("2024-03-28", "Jueves Santo"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-03-31", "Pascua"),
            ("2024-05-01", "Día de la Agricultura y el Trabajo"),
            ("2024-05-09", "Ascensión"),
            ("2024-05-18", "Fiesta de la Bandera y la Universidad"),
            ("2024-05-23", "Día de la Soberanía Nacional"),
            ("2024-05-30", "Fête-Dieu"),
            ("2024-06-27", "Fiesta de Nuestra Señora del Perpetuo Socorro, patrona de Haití"),
            ("2024-08-15", "Asunción de María"),
            ("2024-09-20", "Aniversario del Nacimiento de Jean-Jacques Dessalines"),
            ("2024-10-17", "Muerte de Dessalines"),
            ("2024-11-01", "Día de Todos los Santos"),
            ("2024-11-02", "Día de Muertos"),
            ("2024-11-18", "Conmemoración de la Batalla de Vertières; Día de las Fuerzas Armadas"),
            ("2024-12-05", "Día del Descubrimiento"),
            ("2024-12-25", "Navidad"),
        )

    def test_l10n_ht(self):
        self.assertLocalizedHolidays(
            "ht",
            ("2024-01-01", "Jounen Endepandans Nasyonal; Nouvèl Ane"),
            ("2024-01-02", "Fèt Zansèt yo"),
            ("2024-02-11", "Kanaval"),
            ("2024-02-12", "Lendi Gras"),
            ("2024-02-13", "Madi Gras"),
            ("2024-02-14", "Mèkredi Sann"),
            ("2024-03-28", "Jedi Sant"),
            ("2024-03-29", "Vandredi Sen"),
            ("2024-03-31", "Pak"),
            ("2024-05-01", "Jounen Agrikilti ak Travay"),
            ("2024-05-09", "Asansyon"),
            ("2024-05-18", "Jounen Drapo ak Inivèsite"),
            ("2024-05-23", "Jounen Nasyonal Souverènte"),
            ("2024-05-30", "Fèt Dye"),
            ("2024-06-27", "Fèt Manman Pèpetyèl Sekou, Patwòn Peyi Dayiti"),
            ("2024-08-15", "Sipozisyon Mari"),
            ("2024-09-20", "Anivèsè Nesans Jean-Jacques Dessalines"),
            ("2024-10-17", "Lanmò Desalin"),
            ("2024-11-01", "Jou tout Sen"),
            ("2024-11-02", "Jou Mouri"),
            ("2024-11-18", "Jounen Fòs Lame; Komemorasyon batay Vertières"),
            ("2024-12-05", "Jounen Dekouvèt"),
            ("2024-12-25", "Nwèl"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.calendars.gregorian import JAN, MAR, APR, MAY, JUN, AUG, SEP, OCT, NOV, DEC
from holidays.countries.australia import Australia
from tests.common import CommonCountryTests


class TestAustralia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Australia, with_subdiv_categories=True)

    def _assertVariableDays(self, year: int, subdiv_holidays_mapping: dict):  # noqa: N802
        for dt, subdivisions in subdiv_holidays_mapping.items():
            dt = date(year, *dt)
            for subdiv, holidays in self.subdiv_holidays.items():  # type: ignore[attr-defined]
                self.assertEqual(
                    dt in holidays,
                    subdiv in subdivisions,
                    f"Failed date `{dt:%Y-%m-%d}`, subdiv `{subdiv}`: {', '.join(subdivisions)}",
                )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, self.full_range)

    def test_australia_day(self):
        name_1801 = "Anniversary Day"
        name_1935 = "Australia Day"
        self.assertHolidayName(name_1935, (f"{year}-01-26" for year in range(1935, self.end_year)))
        self.assertNoHolidayName(name_1935, range(self.start_year, 1935))
        self.assertNoHoliday(f"{year}-01-26" for year in range(self.start_year, 1935))

        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "NSW":
                    self.assertHolidayName(name_1801, holidays, range(1888, 1946))
                    self.assertHolidayName(name_1935, holidays, range(1946, self.end_year))
                    self.assertNoHolidayName(
                        name_1801,
                        holidays,
                        range(self.start_year, 1888),
                        range(1946, self.end_year),
                    )
                    self.assertNoHolidayName(name_1935, holidays, range(self.start_year, 1946))
                case "SA":
                    self.assertHolidayName(name_1935, holidays, range(1935, self.end_year))
                    self.assertNoHolidayName(name_1935, holidays, range(self.start_year, 1935))
                    self.assertNoHolidayName(name_1801, holidays)
                case _:
                    self.assertHolidayName(name_1801, holidays, range(1888, 1935))
                    self.assertHolidayName(name_1935, holidays, range(1935, self.end_year))
                    self.assertNoHolidayName(
                        name_1801,
                        holidays,
                        range(self.start_year, 1888),
                        range(1935, self.end_year),
                    )
                    self.assertNoHolidayName(name_1935, holidays, range(self.start_year, 1935))

    def test_good_friday(self):
        name = "Good Friday"
        dts = (
            "1999-04-02",
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, self.full_range)

    def test_easter_saturday(self):
        name = "Easter Saturday"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "VIC":
                    self.assertHolidayName(name, holidays, range(2003, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2003))
                case "TAS" | "WA":
                    self.assertNoHolidayName(name, holidays)
                case _:
                    self.assertHolidayName(name, holidays, self.full_range)

    def test_easter_sunday(self):
        name = "Easter Sunday"
        self.assertNoHolidayName(name)

        subdiv_start_years = {
            "ACT": 2016,
            "NSW": 2011,
            "NT": 2024,
            "QLD": 2017,
            "SA": 2024,
            "VIC": 2016,
            "WA": 2022,
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            if start_year := subdiv_start_years.get(subdiv):
                self.assertHolidayName(name, holidays, range(start_year, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, start_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_easter_monday(self):
        name = "Easter Monday"
        dts = (
            "1999-04-05",
            "2000-04-24",
            "2010-04-05",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, self.full_range)

    def test_anzac_day(self):
        name = "ANZAC Day"
        self.assertHolidayName(name, (f"{year}-04-25" for year in range(1921, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1921))
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name, holidays, range(1921, self.end_year))
            self.assertNoHolidayName(name, holidays, range(self.start_year, 1921))
            match subdiv:
                case "ACT":
                    self.assertHolidayName(name, holidays, "2026-04-25")
                    self.assertHolidayName(f"{name} (observed)", holidays, "2026-04-27")
                case "NSW" | "WA":
                    self.assertHolidayName(name, holidays, "2026-04-25")
                    self.assertHolidayName(
                        f"{name} (observed)",
                        holidays,
                        "2026-04-27",
                        "2027-04-26",
                    )

    def test_labor_day(self):
        name = "Labour Day"
        self.assertNoHolidayName(name)

        names = {"NT": "May Day", "TAS": "Eight Hours Day"}
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(names.get(subdiv, name), holidays, self.full_range)

    def test_sovereigns_birthday(self):
        name_king = "King's Birthday"
        name_queen = "Queen's Birthday"
        self.assertHolidayName(
            name_king,
            (f"{year}-11-09" for year in range(1902, 1912)),
            (f"{year}-06-03" for year in range(1912, 1936)),
        )

        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(
                name_king,
                holidays,
                (f"{year}-11-09" for year in range(1902, 1912)),
                (f"{year}-06-03" for year in range(1912, 1936)),
            )
        self.assertNoHolidayName(name_king, range(1936, self.end_year))
        self.assertNoHolidayName(name_queen, range(1936, self.end_year))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, self.full_range)

    def test_boxing_day(self):
        name_common = "Boxing Day"
        name_sa = "Proclamation Day"
        self.assertHolidayName(name_common, (f"{year}-12-26" for year in self.full_range))

        for subdiv, holidays in self.subdiv_holidays.items():
            name = name_sa if subdiv == "SA" else name_common
            self.assertHolidayName(name, holidays, self.full_range)

    def test_canberra_day(self):
        name = "Canberra Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "ACT":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-12" for year in range(1913, 1959))
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    "1959-03-16",
                    "2000-03-20",
                    "2007-03-19",
                    "2008-03-10",
                    "2010-03-08",
                    "2012-03-12",
                    "2018-03-12",
                    "2019-03-11",
                    "2020-03-09",
                    "2021-03-08",
                    "2022-03-14",
                    "2023-03-13",
                    "2024-03-11",
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1913))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_family_and_community_day(self):
        name = "Family & Community Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "ACT":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2010-09-26",
                    "2011-10-10",
                    "2012-10-08",
                    "2013-09-30",
                    "2014-09-29",
                    "2015-09-28",
                    "2016-09-26",
                    "2017-09-25",
                )
                self.assertNoHolidayName(
                    name, holidays, range(self.start_year, 2010), range(2018, self.end_year)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_reconciliation_day(self):
        name = "Reconciliation Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "ACT":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2018-05-28",
                    "2019-05-27",
                    "2020-06-01",
                    "2021-05-31",
                    "2022-05-30",
                    "2023-05-29",
                    "2024-05-27",
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2018))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_bank_holiday(self):
        name = "Bank Holiday"
        self.assertNoHolidayName(name)

        self.assertSubdivNswBankHolidayName(
            name,
            "2018-08-06",
            "2019-08-05",
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NSW":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2000-08-07",
                    "2007-08-06",
                    "2008-08-04",
                    "2009-08-03",
                    "2010-08-02",
                )
                self.assertNoHolidayName(
                    name, holidays, range(self.start_year, 1912), range(2011, self.end_year)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_picnic_day(self):
        name = "Picnic Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "NT":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2000-08-07",
                    "2010-08-02",
                    "2018-08-06",
                    "2019-08-05",
                    "2020-08-03",
                    "2021-08-02",
                    "2022-08-01",
                    "2023-08-07",
                    "2024-08-05",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_royal_queensland_show(self):
        name = "The Royal Queensland Show"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "QLD":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2000-08-16",
                    "2010-08-11",
                    "2018-08-15",
                    "2019-08-14",
                    "2020-08-14",
                    "2021-10-29",
                    "2022-08-10",
                    "2023-08-16",
                    "2024-08-14",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_adelaide_cup_day(self):
        name = "Adelaide Cup Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SA":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2000-05-15",
                    "2005-05-16",
                    "2006-03-13",
                    "2018-03-12",
                    "2019-03-11",
                    "2020-03-09",
                    "2021-03-08",
                    "2022-03-14",
                    "2023-03-13",
                    "2024-03-11",
                )
                self.assertHolidayName(name, holidays, range(1973, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1973))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_melbourne_cup_day(self):
        name = "Melbourne Cup Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "VIC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2009-11-03",
                    "2010-11-02",
                    "2018-11-06",
                    "2019-11-05",
                    "2020-11-03",
                    "2021-11-02",
                    "2022-11-01",
                    "2023-11-07",
                    "2024-11-05",
                )
                self.assertHolidayName(name, holidays, range(2009, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_grand_final_day(self):
        name = "Grand Final Day"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "VIC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2015-10-02",
                    "2016-09-30",
                    "2017-09-29",
                    "2018-09-28",
                    "2019-09-27",
                    "2020-10-23",
                    "2021-09-24",
                    "2022-09-23",
                    "2023-09-29",
                    "2024-09-27",
                )
                self.assertHolidayName(name, holidays, range(2015, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2015))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_western_australia_day(self):
        name_1833 = "Foundation Day"
        name_2012 = "Western Australia Day"
        self.assertNoHolidayName(name_1833)
        self.assertNoHolidayName(name_2012)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "WA":
                self.assertHolidayName(
                    name_1833,
                    holidays,
                    "2000-06-05",
                    "2005-06-06",
                    "2010-06-07",
                    "2011-06-06",
                )
                self.assertHolidayName(
                    name_2012,
                    holidays,
                    "2012-06-04",
                    "2013-06-03",
                    "2014-06-02",
                    "2015-06-01",
                    "2016-06-06",
                    "2017-06-05",
                    "2018-06-04",
                    "2019-06-03",
                    "2020-06-01",
                    "2021-06-07",
                    "2022-06-06",
                )
                self.assertNoHolidayName(name_1833, holidays, range(2012, self.end_year))
                self.assertNoHolidayName(name_2012, holidays, range(self.start_year, 2012))
            else:
                self.assertNoHolidayName(name_1833, holidays)
                self.assertNoHolidayName(name_2012, holidays)
        self.assertNoSubdivWaHolidayName(name_1833, 1832)

    def test_national_day_of_mourning_for_queen_elizabeth_ii(self):
        name = "National Day of Mourning for Queen Elizabeth II"
        dts = "2022-09-22"
        self.assertHolidayName(name, dts)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)

    def test_special_holidays(self):
        self.assertSubdivActHoliday("2020-04-20", "2021-04-25")
        self.assertSubdivQldHoliday("2010-12-28", "2011-01-03", "2012-06-11")
        self.assertNoSubdivQldNonObservedHoliday("2010-12-28", "2011-01-03")
        self.assertSubdivWaHoliday("2011-04-26")

    def test_christmas_eve(self):
        for subdiv, name, start_year in (
            ("NT", "Christmas Eve (from 7pm)", 2016),
            ("QLD", "Christmas Eve (from 6pm)", 2019),
            ("SA", "Christmas Eve (from 7pm)", 2012),
        ):
            self.assertNoHolidayName(name)
            self.assertHalfDayHolidayName(
                name,
                self.subdiv_half_day_holidays[subdiv],
                (f"{year}-12-24" for year in range(start_year, self.end_year)),
            )
            self.assertNoHalfDayHolidayName(
                name,
                self.subdiv_half_day_holidays[subdiv],
                (f"{year}-12-24" for year in range(self.start_year, start_year)),
            )

    def test_all_holidays(self):
        holidays_found = set()
        for subdiv in Australia.subdivisions:
            holidays_found.update(
                Australia(
                    categories=Australia.supported_categories,
                    subdiv=subdiv,
                    observed=False,
                    years=(1930, 1957, 2012, 2015, 2023),
                ).values()
            )
        all_holidays = {
            "New Year's Day",
            "Anniversary Day",
            "Australia Day",
            "Adelaide Cup Day",
            "Canberra Day",
            "Good Friday",
            "Easter Saturday",
            "Easter Sunday",
            "Easter Monday",
            "Easter Tuesday",
            "ANZAC Day",
            "Reconciliation Day",
            "Queen's Birthday",
            "Queen's Diamond Jubilee",
            "King's Birthday",
            "Bank Holiday",
            "The Royal Queensland Show",
            "Western Australia Day",
            "Foundation Day",
            "Family & Community Day",
            "Labour Day",
            "Eight Hours Day",
            "May Day",
            "Picnic Day",
            "Melbourne Cup Day",
            "Grand Final Day",
            "Christmas Day",
            "Proclamation Day",
            "Boxing Day",
            "Christmas Eve (from 6pm)",
            "Christmas Eve (from 7pm)",
            "New Year's Eve (from 7pm)",
        }
        self.assertEqual(all_holidays, holidays_found)

    def test_holidays_2019(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 28): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 4): {"WA"},
            (MAR, 11): {"ACT", "SA", "TAS", "VIC"},
            (APR, 19): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 20): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (APR, 21): {"ACT", "NSW", "QLD", "VIC"},
            (APR, 22): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAY, 6): {"NT", "QLD"},
            (MAY, 27): {"ACT"},
            (JUN, 3): {"WA"},
            (JUN, 10): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 5): {"NT"},
            (AUG, 14): {"QLD"},
            (SEP, 27): {"VIC"},
            (SEP, 30): {"WA"},
            (OCT, 7): {"ACT", "NSW", "QLD", "SA"},
            (NOV, 5): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2019, subdivision_days)

    def test_holidays_2020(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 26): {"SA"},
            (JAN, 27): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 2): {"WA"},
            (MAR, 9): {"ACT", "SA", "TAS", "VIC"},
            (APR, 10): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 11): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (APR, 12): {"ACT", "NSW", "QLD", "VIC"},
            (APR, 13): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 27): {"WA"},
            (MAY, 4): {"NT", "QLD"},
            (JUN, 1): {"ACT", "WA"},
            (JUN, 8): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 3): {"NT"},
            (AUG, 14): {"QLD"},
            (SEP, 28): {"WA"},
            (OCT, 5): {"ACT", "NSW", "QLD", "SA"},
            (OCT, 23): {"VIC"},
            (NOV, 3): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "QLD", "VIC", "WA"},
            (DEC, 28): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2020, subdivision_days)

    def test_holidays_2021(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 1): {"WA"},
            (MAR, 8): {"ACT", "SA", "TAS", "VIC"},
            (APR, 2): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 3): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (APR, 4): {"ACT", "NSW", "QLD", "VIC"},
            (APR, 5): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "SA", "TAS", "VIC", "WA"},
            (APR, 26): {"ACT", "NT", "QLD", "SA", "WA"},
            (MAY, 3): {"NT", "QLD"},
            (MAY, 31): {"ACT"},
            (JUN, 7): {"WA"},
            (JUN, 14): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 2): {"NT"},
            (SEP, 24): {"VIC"},
            (SEP, 27): {"WA"},
            (OCT, 4): {"ACT", "NSW", "QLD", "SA"},
            (OCT, 29): {"QLD"},
            (NOV, 2): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "QLD", "SA", "VIC", "WA"},
            (DEC, 27): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 28): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2021, subdivision_days)

    def test_holidays_2022(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "VIC", "WA"},
            (JAN, 3): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 7): {"WA"},
            (MAR, 14): {"ACT", "SA", "TAS", "VIC"},
            (APR, 15): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 16): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (APR, 17): {"ACT", "NSW", "QLD", "VIC", "WA"},
            (APR, 18): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAY, 2): {"NT", "QLD"},
            (MAY, 30): {"ACT"},
            (JUN, 6): {"WA"},
            (JUN, 13): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 1): {"NT"},
            (AUG, 10): {"QLD"},
            (SEP, 22): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (SEP, 23): {"VIC"},
            (SEP, 26): {"WA"},
            (OCT, 3): {"ACT", "NSW", "QLD", "SA"},
            (NOV, 1): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 27): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2022, subdivision_days)

    def test_holidays_2023(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "SA", "VIC", "WA"},
            (JAN, 2): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 6): {"WA"},
            (MAR, 13): {"ACT", "SA", "TAS", "VIC"},
            (APR, 7): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 8): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (APR, 9): {"ACT", "NSW", "QLD", "VIC", "WA"},
            (APR, 10): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAY, 1): {"NT", "QLD"},
            (MAY, 29): {"ACT"},
            (JUN, 5): {"WA"},
            (JUN, 12): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 7): {"NT"},
            (AUG, 16): {"QLD"},
            (SEP, 29): {"VIC"},
            (SEP, 25): {"WA"},
            (OCT, 2): {"ACT", "NSW", "QLD", "SA"},
            (NOV, 7): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2023, subdivision_days)

    def test_holidays_2024(self):
        subdivision_days = {
            (JAN, 1): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (JAN, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 4): {"WA"},
            (MAR, 11): {"ACT", "SA", "TAS", "VIC"},
            (MAR, 29): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAR, 30): {"ACT", "NSW", "NT", "QLD", "SA", "VIC"},
            (MAR, 31): {"ACT", "NSW", "NT", "QLD", "SA", "VIC", "WA"},
            (APR, 1): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (APR, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (MAY, 6): {"NT", "QLD"},
            (MAY, 27): {"ACT"},
            (JUN, 3): {"WA"},
            (JUN, 10): {"ACT", "NSW", "NT", "SA", "TAS", "VIC"},
            (AUG, 5): {"NT"},
            (AUG, 14): {"QLD"},
            (SEP, 27): {"VIC"},
            (SEP, 23): {"WA"},
            (OCT, 7): {"ACT", "NSW", "QLD", "SA"},
            (NOV, 5): {"VIC"},
            (DEC, 25): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
            (DEC, 26): {"ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"},
        }
        self._assertVariableDays(2024, subdivision_days)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day; New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-03-07", "Labour Day"),
            ("2022-03-14", "Adelaide Cup Day; Canberra Day; Eight Hours Day; Labour Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Easter Saturday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-05-02", "Labour Day; May Day"),
            ("2022-05-30", "Reconciliation Day"),
            ("2022-06-06", "Western Australia Day"),
            ("2022-06-13", "Queen's Birthday"),
            ("2022-08-01", "Bank Holiday; Picnic Day"),
            ("2022-08-10", "The Royal Queensland Show"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-09-23", "Grand Final Day"),
            ("2022-09-26", "Queen's Birthday"),
            ("2022-10-03", "Labour Day; Queen's Birthday"),
            ("2022-11-01", "Melbourne Cup Day"),
            ("2022-12-24", "Christmas Eve (from 6pm); Christmas Eve (from 7pm)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed); Proclamation Day"),
            (
                "2022-12-27",
                "Boxing Day; Boxing Day (observed); Christmas Day (observed); "
                "Proclamation Day (observed)",
            ),
            ("2022-12-31", "New Year's Eve (from 7pm)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day; New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-03-07", "Labor Day"),
            ("2022-03-14", "Adelaide Cup Day; Canberra Day; Eight Hours Day; Labor Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Easter Saturday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-05-02", "Labor Day; May Day"),
            ("2022-05-30", "Reconciliation Day"),
            ("2022-06-06", "Western Australia Day"),
            ("2022-06-13", "Queen's Birthday"),
            ("2022-08-01", "Bank Holiday; Picnic Day"),
            ("2022-08-10", "The Royal Queensland Show"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-09-23", "Grand Final Day"),
            ("2022-09-26", "Queen's Birthday"),
            ("2022-10-03", "Labor Day; Queen's Birthday"),
            ("2022-11-01", "Melbourne Cup Day"),
            ("2022-12-24", "Christmas Eve (from 6pm); Christmas Eve (from 7pm)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed); Proclamation Day"),
            (
                "2022-12-27",
                "Boxing Day; Boxing Day (observed); Christmas Day (observed); "
                "Proclamation Day (observed)",
            ),
            ("2022-12-31", "New Year's Eve (from 7pm)"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-03", "ชดเชยวันขึ้นปีใหม่; วันขึ้นปีใหม่"),
            ("2022-01-26", "วันชาติออสเตรเลีย"),
            ("2022-03-07", "วันแรงงาน"),
            ("2022-03-14", "วันแคนเบอร์รา; วันแปดชั่วโมง (วันแรงงาน); วันแรงงาน; วันแอดิเลดคัพ"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-16", "วันเสาร์อีสเตอร์"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-04-25", "วันแอนแซค"),
            ("2022-05-02", "วันเมย์เดย์ (วันแรงงาน); วันแรงงาน"),
            ("2022-05-30", "วันแห่งการปรองดอง"),
            ("2022-06-06", "วันเวสเทิร์นออสเตรเลีย"),
            ("2022-06-13", "วันเฉลิมพระชนมพรรษาสมเด็จพระราชินีนาถ"),
            ("2022-08-01", "วันปิกนิก; วันหยุดธนาคาร"),
            ("2022-08-10", "เทศกาลรอยัลควีนส์แลนด์โชว์"),
            ("2022-09-22", "วันไว้ทุกข์แห่งชาติแด่สมเด็จพระราชินีนาถเอลิซาเบธที่ 2"),
            ("2022-09-23", "วันศุกร์ก่อนวันแข่งฟุตบอลออสเตรเลีย (AFL) รอบสุดท้าย"),
            ("2022-09-26", "วันเฉลิมพระชนมพรรษาสมเด็จพระราชินีนาถ"),
            ("2022-10-03", "วันเฉลิมพระชนมพรรษาสมเด็จพระราชินีนาถ; วันแรงงาน"),
            ("2022-11-01", "วันเมลเบิร์นคัพ"),
            ("2022-12-24", "วันคริสต์มาสอีฟ (ตั้งแต่ 18:00 น.); วันคริสต์มาสอีฟ (ตั้งแต่ 19:00 น.)"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส; วันสถาปนา; วันเปิดกล่องของขวัญ"),
            ("2022-12-27", "ชดเชยวันคริสต์มาส; ชดเชยวันสถาปนา; ชดเชยวันเปิดกล่องของขวัญ; วันเปิดกล่องของขวัญ"),
            ("2022-12-31", "วันสิ้นปี (ตั้งแต่ 19:00 น.)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.liberia import Liberia
from tests.common import CommonCountryTests


class TestLiberia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Liberia)

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_inauguration_day(self):
        name = "Inauguration Day"
        years_after_election = {1960, 1964, 1968, 1972, 1976, 1986, 1998, 2006, 2012, 2018, 2024}
        self.assertHolidayName(
            name,
            "2006-01-02",
            "2012-01-02",
            "2018-01-01",
            "2024-01-01",
        )
        self.assertHolidayName(name, years_after_election)
        self.assertNoHolidayName(name, set(self.full_range) - years_after_election)

    def test_armed_forces_day(self):
        self.assertHolidayName("Armed Forces Day", (f"{year}-02-11" for year in self.full_range))

    def test_decoration_day(self):
        name = "Decoration Day"
        self.assertHolidayName(
            name,
            "2020-03-11",
            "2021-03-10",
            "2022-03-09",
            "2023-03-08",
            "2024-03-13",
            "2025-03-12",
        )
        self.assertHolidayName(name, self.full_range)

    def test_jj_roberts_memorial_day(self):
        self.assertHolidayName(
            "J. J. Roberts Memorial Birthday", (f"{year}-03-15" for year in self.full_range)
        )

    def test_fasting_and_prayer_day(self):
        name = "Fasting and Prayer Day"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-09",
            "2022-04-08",
            "2023-04-14",
            "2024-04-12",
            "2025-04-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_unification_and_integration_day(self):
        name = "National Unification and Integration Day"
        self.assertHolidayName(name, (f"{year}-05-14" for year in range(1960, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1960))

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-07-26" for year in self.full_range))

    def test_national_flag_day(self):
        self.assertHolidayName("National Flag Day", (f"{year}-08-24" for year in self.full_range))

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertHolidayName(
            name,
            "2020-11-05",
            "2021-11-04",
            "2022-11-03",
            "2023-11-02",
            "2024-11-07",
            "2025-11-06",
        )
        self.assertHolidayName(name, self.full_range)

    def test_tubman_administration_goodwill_day(self):
        self.assertHolidayName(
            "Tubman Administration Goodwill Day", (f"{year}-11-29" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-02-11", "Armed Forces Day"),
            ("2025-03-12", "Decoration Day"),
            ("2025-03-15", "J. J. Roberts Memorial Birthday"),
            ("2025-04-11", "Fasting and Prayer Day"),
            ("2025-05-14", "National Unification and Integration Day"),
            ("2025-07-26", "Independence Day"),
            ("2025-08-24", "National Flag Day"),
            ("2025-11-06", "Thanksgiving Day"),
            ("2025-11-29", "Tubman Administration Goodwill Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.trinidad_and_tobago import TrinidadAndTobago
from tests.common import CommonCountryTests


class TestTrinidadAndTobago(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(TrinidadAndTobago)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2018-04-03",
            "2029-04-03",
            "2040-04-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2043-03-31",
            "2054-03-31",
            "2065-03-31",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_spiritual_baptist_liberation_day(self):
        name = "Spiritual Baptist Liberation Day"
        self.assertHolidayName(name, (f"{year}-03-30" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, range(1963, 1996))
        obs_dts = (
            "2003-03-31",
            "2008-03-31",
            "2014-03-31",
            "2018-04-03",
            "2025-04-01",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_indian_arrival_day(self):
        name = "Indian Arrival Day"
        self.assertHolidayName(name, (f"{year}-05-30" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, range(1963, 1996))
        obs_dts = (
            "2004-05-31",
            "2010-05-31",
            "2013-05-31",
            "2021-05-31",
            "2024-05-31",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2013-05-31",
            "2014-06-20",
            "2024-05-31",
            "2025-06-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-06-19" for year in range(1973, 2050)))
        self.assertNoHolidayName(name, range(1963, 1973))
        obs_dts = (
            "2005-06-20",
            "2011-06-20",
            "2014-06-20",
            "2016-06-20",
            "2022-06-20",
            "2025-06-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_african_emancipation_day(self):
        name = "African Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in range(1985, 2050)))
        self.assertNoHolidayName(name, range(1963, 1985))
        obs_dts = (
            "2004-08-02",
            "2010-08-02",
            "2021-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-08-31" for year in self.full_range))
        obs_dts = (
            "2003-09-01",
            "2008-09-01",
            "2014-09-01",
            "2025-09-01",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_republic_day(self):
        name = "Republic Day"
        self.assertHolidayName(name, (f"{year}-09-24" for year in range(1976, 2050)))
        self.assertNoHolidayName(name, range(1963, 1976))
        obs_dts = (
            "2000-09-25",
            "2006-09-25",
            "2017-09-25",
            "2023-09-25",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_diwali(self):
        name = "Divali"
        self.assertHolidayName(
            name,
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
            "2025-10-20",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2019-10-28",
            "2023-11-13",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Eid-Ul-Fitr"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2009-09-21",
            "2012-08-20",
            "2020-05-25",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival_monday(self):
        name = "Carnival Monday"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Carnival Tuesday"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_optional_2025(self):
        self.assertOptionalHolidaysInYear(
            2025,
            ("2025-03-03", "Carnival Monday"),
            ("2025-03-04", "Carnival Tuesday"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-30", "Spiritual Baptist Liberation Day"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Eid-Ul-Fitr"),
            ("2024-05-30", "Corpus Christi; Indian Arrival Day"),
            ("2024-05-31", "Corpus Christi (observed); Indian Arrival Day (observed)"),
            ("2024-06-19", "Labour Day"),
            ("2024-08-01", "African Emancipation Day"),
            ("2024-08-31", "Independence Day"),
            ("2024-09-24", "Republic Day"),
            ("2024-10-31", "Divali"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-03-30", "Spiritual Baptist Liberation Day"),
            ("2025-03-31", "Eid-Ul-Fitr"),
            ("2025-04-01", "Spiritual Baptist Liberation Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-30", "Indian Arrival Day"),
            ("2025-06-19", "Corpus Christi; Labour Day"),
            ("2025-06-20", "Corpus Christi (observed); Labour Day (observed)"),
            ("2025-08-01", "African Emancipation Day"),
            ("2025-08-31", "Independence Day"),
            ("2025-09-01", "Independence Day (observed)"),
            ("2025-09-24", "Republic Day"),
            ("2025-10-20", "Divali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-03-03", "Carnival Monday"),
            ("2025-03-04", "Carnival Tuesday"),
            ("2025-03-30", "Spiritual Baptist Liberation Day"),
            ("2025-03-31", "Eid-Ul-Fitr"),
            ("2025-04-01", "Spiritual Baptist Liberation Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-30", "Indian Arrival Day"),
            ("2025-06-19", "Corpus Christi; Labour Day"),
            ("2025-06-20", "Corpus Christi (observed); Labour Day (observed)"),
            ("2025-08-01", "African Emancipation Day"),
            ("2025-08-31", "Independence Day"),
            ("2025-09-01", "Independence Day (observed)"),
            ("2025-09-24", "Republic Day"),
            ("2025-10-20", "Divali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-03", "Carnival Monday"),
            ("2025-03-04", "Carnival Tuesday"),
            ("2025-03-30", "Spiritual Baptist Liberation Day"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-01", "Spiritual Baptist Liberation Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-30", "Indian Arrival Day"),
            ("2025-06-19", "Corpus Christi; Labor Day"),
            ("2025-06-20", "Corpus Christi (observed); Labor Day (observed)"),
            ("2025-08-01", "African Emancipation Day"),
            ("2025-08-31", "Independence Day"),
            ("2025-09-01", "Independence Day (observed)"),
            ("2025-09-24", "Republic Day"),
            ("2025-10-20", "Diwali"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.eritrea import Eritrea
from tests.common import CommonCountryTests


class TestEritrea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Eritrea)

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_orthodox_christmas(self):
        self.assertHolidayName("Orthodox Christmas", (f"{year}-01-07" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName(
            "Epiphany",
            (f"{year}-01-19" for year in self.full_range if year % 4 != 0),
            (f"{year}-01-20" for year in self.full_range if year % 4 == 0),
        )

    def test_womens_day(self):
        self.assertHolidayName("Women's Day", (f"{year}-03-08" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_easter(self):
        name = "Orthodox Easter"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_international_workers_day(self):
        self.assertHolidayName(
            "International Workers' Day", (f"{year}-05-01" for year in self.full_range)
        )

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-05-24" for year in self.full_range))

    def test_martyrs_day(self):
        self.assertHolidayName("Martyrs' Day", (f"{year}-06-20" for year in self.full_range))

    def test_revolution_day(self):
        self.assertHolidayName("Revolution Day", (f"{year}-09-01" for year in self.full_range))

    def test_ethiopian_new_year(self):
        self.assertHolidayName(
            "Ethiopian New Year",
            (f"{year}-09-11" for year in self.full_range if year % 4 != 3),
            (f"{year}-09-12" for year in self.full_range if year % 4 == 3),
        )

    def test_finding_of_the_true_cross(self):
        self.assertHolidayName(
            "Finding of the True Cross",
            (f"{year}-09-27" for year in self.full_range if year % 4 != 3),
            (f"{year}-09-28" for year in self.full_range if year % 4 == 3),
        )

    def test_prophets_birthday(self):
        name = "Prophet's Birthday"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_fenkil_day(self):
        name = "Fenkil Day"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-02-10" for year in self.full_range))

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-01-07", "Orthodox Christmas"),
            ("2020-01-20", "Epiphany"),
            ("2020-03-08", "Women's Day"),
            ("2020-04-17", "Good Friday"),
            ("2020-04-19", "Orthodox Easter"),
            ("2020-05-01", "International Workers' Day"),
            ("2020-05-24", "Eid al-Fitr (estimated); Independence Day"),
            ("2020-06-20", "Martyrs' Day"),
            ("2020-07-31", "Eid al-Adha (estimated)"),
            ("2020-09-01", "Revolution Day"),
            ("2020-09-11", "Ethiopian New Year"),
            ("2020-09-27", "Finding of the True Cross"),
            ("2020-10-29", "Prophet's Birthday (estimated)"),
        )

    def test_2020_government(self):
        self.assertGovernmentHolidaysInYear(
            2020,
            ("2020-02-10", "Fenkil Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.ivory_coast import IvoryCoast
from tests.common import CommonCountryTests


class TestIvoryCoast(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(IvoryCoast)

    def test_special_holidays(self):
        self.assertHoliday(
            "2010-10-29",
            "2024-02-12",
        )

    def test_new_years_day(self):
        self.assertHolidayName("1er janvier", (f"{year}-01-01" for year in self.full_range))

    def test_labor_day(self):
        name = "Fête du travail"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
        )
        self.assertHolidayName(f"Lendemain de la {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Fête Nationale"
        self.assertHolidayName(name, (f"{year}-08-07" for year in self.full_range))
        obs_dts = (
            "2005-08-08",
            "2011-08-08",
            "2016-08-08",
            "2022-08-08",
        )
        self.assertHolidayName(f"Lendemain de la {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_peace_day(self):
        self.assertHolidayName(
            "Journée Nationale de la Paix", (f"{year}-11-15" for year in self.full_range)
        )

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_ascension_day(self):
        name = "Jour de l'Ascension"
        self.assertHolidayName(
            name,
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName(
            "Fête de l'Assomption", (f"{year}-08-15" for year in self.full_range)
        )

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Fête de la Toussaint", (f"{year}-11-01" for year in self.full_range)
        )

    def test_anniversary_of_death_of_first_president(self):
        name = "Anniversaire du décès du Président Felix Houphouet-Boigny"
        self.assertHolidayName(
            name,
            "1997-12-07",
            "1998-12-07",
            "1999-12-07",
            "2000-12-07",
        )
        self.assertNoHolidayName(name, range(2001, self.end_year))

    def test_christmas_day(self):
        name = "Fête de Noël"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"Lendemain de la {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Fête de fin du Ramadan"
        self.assertHolidayName(
            name,
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2012-08-20",
            "2017-06-26",
            "2020-05-25",
            "2025-03-31",
        )
        self.assertHolidayName(f"Lendemain de la {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Fête de la Tabaski"
        self.assertHolidayName(
            name,
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.full_range)
        obs_dts = (
            "2016-09-12",
            "2019-08-12",
            "2024-06-17",
        )
        self.assertHolidayName(f"Lendemain de la {name}", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_after_mawlid(self):
        name = "Lendemain de l'Anniversaire de la Naissance du Prophète Mahomet"
        self.assertHolidayName(
            name,
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_day_after_laylat_al_qadr(self):
        name = "Lendemain de la Nuit du Destin"
        self.assertHolidayName(
            name,
            "2021-05-09",
            "2022-04-28",
            "2023-04-18",
            "2024-04-06",
            "2025-03-27",
        )
        self.assertHolidayName(name, self.full_range)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "1er janvier"),
            ("2024-02-12", "Victoire à la Coupe d'Afrique des Nations 2024"),
            ("2024-04-06", "Lendemain de la Nuit du Destin"),
            ("2024-04-10", "Fête de fin du Ramadan"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-05-01", "Fête du travail"),
            ("2024-05-09", "Jour de l'Ascension"),
            ("2024-05-20", "Lundi de Pentecôte"),
            ("2024-06-16", "Fête de la Tabaski"),
            ("2024-06-17", "Lendemain de la Fête de la Tabaski"),
            ("2024-08-07", "Fête Nationale"),
            ("2024-08-15", "Fête de l'Assomption"),
            ("2024-09-15", "Lendemain de l'Anniversaire de la Naissance du Prophète Mahomet"),
            ("2024-11-01", "Fête de la Toussaint"),
            ("2024-11-15", "Journée Nationale de la Paix"),
            ("2024-12-25", "Fête de Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "1er janvier"),
            ("2025-03-27", "Lendemain de la Nuit du Destin"),
            ("2025-03-30", "Fête de fin du Ramadan"),
            ("2025-03-31", "Lendemain de la Fête de fin du Ramadan"),
            ("2025-04-21", "Lundi de Pâques"),
            ("2025-05-01", "Fête du travail"),
            ("2025-05-29", "Jour de l'Ascension"),
            ("2025-06-06", "Fête de la Tabaski"),
            ("2025-06-09", "Lundi de Pentecôte"),
            ("2025-08-07", "Fête Nationale"),
            ("2025-08-15", "Fête de l'Assomption"),
            ("2025-09-04", "Lendemain de l'Anniversaire de la Naissance du Prophète Mahomet"),
            ("2025-11-01", "Fête de la Toussaint"),
            ("2025-11-15", "Journée Nationale de la Paix"),
            ("2025-12-25", "Fête de Noël"),
        )

    def test_l10n_en_ci(self):
        self.assertLocalizedHolidays(
            "en_CI",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-27", "Day after Lailatou-Kadr"),
            ("2025-03-30", "Aid-El-Fitr"),
            ("2025-03-31", "Day after the Aid-El-Fitr"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-06-06", "Aid-El-Kebir"),
            ("2025-06-09", "Whit Monday"),
            ("2025-08-07", "Independence Day"),
            ("2025-08-15", "Assumption Day"),
            ("2025-09-04", "Day after Maouloud"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-15", "National Peace Day"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-27", "Day after Night of Power"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-03-31", "Day after the Eid al-Fitr"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-06-09", "Whit Monday"),
            ("2025-08-07", "Independence Day"),
            ("2025-08-15", "Assumption Day"),
            ("2025-09-04", "Day after Prophet's Birthday"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-15", "National Peace Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, APR, MAY, JUN, JUL, SEP, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_MON


class Latvia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Latvia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Latvia>
        * <https://web.archive.org/web/20250210170838/https://www.information.lv/>
        * [Law "On Public Holidays, Commemoration Days, and Celebration Days" of Oct 3, 1990](https://web.archive.org/web/20260308023501/https://likumi.lv/doc.php?id=72608)
        * [Law of Sep 21, 1995](https://web.archive.org/web/20251217210932/https://likumi.lv/ta/id/37110-grozijumi-likuma-par-svetku-un-atceres-dienam-)
        * [Law of Dec 18, 1996](https://web.archive.org/web/20251120000352/https://likumi.lv/ta/id/41773-grozijumi-likuma-par-svetku-un-atceres-dienam-)
        * [Law of Mar 21, 2002](https://web.archive.org/web/20250918022517/https://likumi.lv/ta/id/60998-grozijumi-likuma-par-svetku-un-atceres-dienam-)
        * [Law of May 24, 2007](https://web.archive.org/web/20251217210804/https://likumi.lv/ta/id/158621-grozijumi-likuma-par-svetku-atceres-un-atzimejamam-dienam-)
        * [Law of Apr 7, 2011](https://web.archive.org/web/20250918032827/https://likumi.lv/ta/id/229236-grozijumi-likuma-par-svetku-atceres-un-atzimejamam-dienam-)

    Regarding observed holidays start year: the law entered into force on May 24, 2007,
    but since May 4 (Restoration of Independence Day) fell on a Friday, starting in 2007 has
    no extra impact.
    """

    country = "LV"
    default_language = "lv"
    # %s (observed).
    observed_label = tr("%s (brīvdiena)")
    supported_languages = ("en_US", "lv", "uk")
    # Law of Oct 3, 1990.
    start_year = 1991

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, LatviaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        # Law of May 24, 2007.
        kwargs.setdefault("observed_since", 2007)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Jaungada diena"))

        # Good Friday.
        self._add_good_friday(tr("Lielā Piektdiena"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Pirmās Lieldienas"))

        # Established by Law of Dec 18, 1996.
        if self._year >= 1997:
            # Easter Monday.
            self._add_easter_monday(tr("Otrās Lieldienas"))

        # Labor Day.
        self._add_labor_day(tr("Darba svētki"))

        # Constitutional Assembly Convocation Day.
        self._add_holiday_may_1(tr("Latvijas Republikas Satversmes sapulces sasaukšanas diena"))

        # Established by Law of Mar 21, 2002.
        # Renamed by Law of Apr 7, 2011 (came into force on May 11, 2011).
        if self._year >= 2002:
            self._add_observed(
                self._add_holiday_may_4(
                    # Restoration of Independence Day.
                    tr("Latvijas Republikas Neatkarības atjaunošanas diena")
                    if self._year >= 2012
                    # Proclamation of Declaration of Independence Day.
                    else tr("Latvijas Republikas Neatkarības deklarācijas pasludināšanas diena")
                )
            )

        # Mother's Day.
        self._add_holiday_2nd_sun_of_may(tr("Mātes diena"))

        # Established by Law of Sep 21, 1995.
        if self._year >= 1996:
            # Whit Sunday.
            self._add_whit_sunday(tr("Vasarsvētki"))

        # Midsummer Eve.
        self._add_holiday_jun_23(tr("Līgo diena"))

        # Midsummer Day.
        self._add_saint_johns_day(tr("Jāņu diena"))

        # Republic of Latvia Proclamation Day.
        self._add_observed(self._add_holiday_nov_18(tr("Latvijas Republikas Proklamēšanas diena")))

        # Established by Law of May 24, 2007.
        if self._year >= 2007:
            # Christmas Eve.
            self._add_christmas_eve(tr("Ziemassvētku vakars"))

        # Christmas Day.
        self._add_christmas_day(tr("Pirmie Ziemassvētki"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Otrie Ziemassvētki"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Vecgada diena"))


class LV(Latvia):
    pass


class LVA(Latvia):
    pass


class LatviaStaticHolidays:
    """Latvia special holidays.

    References:
        * [Law of Oct 2, 2014](https://web.archive.org/web/20251217210815/https://likumi.lv/ta/id/269373-grozijumi-likuma-par-svetku-atceres-un-atzimejamam-dienam-)
        * [Law of Jun 21, 2018](https://web.archive.org/web/20251217210926/https://likumi.lv/ta/id/299942-grozijums-likuma-par-svetku-atceres-un-atzimejamam-dienam-)
        * [Law of May 28, 2023](https://web.archive.org/web/20251217210837/https://likumi.lv/ta/id/342196-grozijumi-likuma-par-svetku-atceres-un-atzimejamam-dienam-)
        * <https://lv.wikipedia.org/wiki/Vispārējie_latviešu_Dziesmu_un_deju_svētki>

    Substituted holidays references:
        * [2001](https://web.archive.org/web/20260403134810/https://likumi.lv/ta/id/56308-par-darbadienas-parcelsanu)
        * [2002](https://web.archive.org/web/20260403134822/https://likumi.lv/ta/id/60769-par-darbdienas-parcelsanu)
        * [2003-2004](https://web.archive.org/web/20260403134847/https://likumi.lv/ta/id/80562-par-darbdienu-parcelsanu)
        * [2004](https://web.archive.org/web/20210506000821/https://likumi.lv/ta/id/83431-par-darbdienu-parcelsanu)
        * [2006](https://web.archive.org/web/20260403134627/https://likumi.lv/ta/id/145144-par-darbdienas-parcelsanu)
        * [2007](https://web.archive.org/web/20260403134617/https://likumi.lv/ta/id/131354-par-darbdienu-parcelsanu)
        * [2008](https://web.archive.org/web/20260403134715/https://likumi.lv/ta/id/173689-par-darbdienu-parcelsanu)
        * [2009](https://web.archive.org/web/20260403134723/https://likumi.lv/ta/id/185051-par-darbdienu-parcelsanu-2009gada)
        * [2010](https://web.archive.org/web/20241104205338/https://likumi.lv/ta/id/203023-par-darbdienu-parcelsanu-2010gada)
        * [2012](https://web.archive.org/web/20250914121202/https://likumi.lv/ta/id/228399-par-darba-dienas-parcelsanu-2012-gada)
        * [2013](https://web.archive.org/web/20260403134736/https://likumi.lv/ta/id/247102-par-darba-dienu-parcelsanu-2013gada)
        * [2014](https://web.archive.org/web/20251113151841/https://likumi.lv/ta/id/257810-par-darba-dienu-parcelsanu-2014-gada)
        * [2015](https://web.archive.org/web/20260403135250/https://likumi.lv/ta/id/266487-par-darba-dienu-parcelsanu-2015gada)
        * [2017](https://web.archive.org/web/20191122133007/https://likumi.lv/ta/id/282928-par-darbadienas-parcelsanu-2017-gada)
        * [2018](https://web.archive.org/web/20260208223051/https://likumi.lv/ta/id/290413-par-darbadienas-parcelsanu-2018-gada)
        * [2020](https://web.archive.org/web/20250620050822/https://likumi.lv/ta/id/308097-par-darbadienas-parcelsanu-2020-gada)
        * [2021](https://web.archive.org/web/20250328150433/https://likumi.lv/ta/id/315837-par-darba-dienu-parcelsanu-2021-gada)
        * [2023](https://web.archive.org/web/20250620155625/https://likumi.lv/ta/id/333470-par-darba-dienas-parcelsanu-2023-gada)
        * [2024](https://web.archive.org/web/20250813123009/https://likumi.lv/ta/id/342680-par-darba-dienu-parcelsanu-2024-gada)
        * [2025](https://web.archive.org/web/20260121064654/https://likumi.lv/ta/id/351684-par-darba-dienu-parcelsanu-2025-gada)
        * [2026](https://web.archive.org/web/20251219143100/https://likumi.lv/ta/id/361161-par-darba-dienu-parcelsanu-2026-gada)
    """

    # Substituted date format.
    substituted_date_format = tr("%d.%m.%Y")

    # Day off (substituted from %s).
    substituted_label = tr("Brīvdiena (pārcelta no %s)")

    #  Nationwide Latvian Song and Dance Celebration Final Day.
    song_and_dance_festival_final_day = tr(
        "Vispārējo latviešu Dziesmu un deju svētku noslēguma diena"
    )

    # Day of the Pastoral Visit of His Holiness Pope Francis to Latvia.
    pope_francis_pastoral_visit_day = tr(
        "Viņa Svētības pāvesta Franciska pastorālās vizītes Latvijā diena"
    )

    # Day of the Latvian Ice Hockey Team's Bronze Medal Win at the 2023 IIHF World Championship.
    hockey_team_win_bronze_medal_day = tr(
        "Diena, kad Latvijas hokeja komanda ieguva bronzas medaļu 2023. gada "
        "Pasaules hokeja čempionātā"
    )

    special_public_holidays = {
        2001: (DEC, 24, DEC, 22),
        2002: (DEC, 30, DEC, 28),
        2003: (NOV, 17, NOV, 15),
        2004: (
            (JAN, 2, JAN, 10),
            (MAY, 3, MAY, 8),
            (JUN, 25, JUN, 19),
            (NOV, 19, NOV, 13),
        ),
        2006: (NOV, 27, DEC, 2),
        2007: (APR, 30, APR, 14),
        2008: (
            (MAY, 2, MAY, 10),
            (NOV, 17, NOV, 22),
        ),
        2009: (
            (JAN, 2, JAN, 10),
            (JUN, 22, JUN, 27),
        ),
        2010: (
            (MAY, 3, MAY, 29),
            (JUN, 25, JUN, 19),
            (NOV, 19, NOV, 13),
        ),
        2012: (APR, 30, APR, 28),
        2013: (
            (DEC, 23, DEC, 14),
            (DEC, 30, DEC, 28),
        ),
        2014: (
            (MAY, 2, MAY, 10),
            (NOV, 17, NOV, 22),
        ),
        2015: (
            (JAN, 2, JAN, 10),
            (JUN, 22, JUN, 27),
        ),
        2017: (MAY, 5, MAY, 13),
        2018: (
            (APR, 30, APR, 21),
            (JUL, 9, song_and_dance_festival_final_day),
            (SEP, 24, pope_francis_pastoral_visit_day),
        ),
        2020: (JUN, 22, JUN, 13),
        2021: (
            (MAY, 3, MAY, 8),
            (JUN, 25, JUN, 19),
            (NOV, 19, NOV, 13),
        ),
        2023: (
            (MAY, 5, MAY, 20),
            (MAY, 29, hockey_team_win_bronze_medal_day),
            (JUL, 10, song_and_dance_festival_final_day),
        ),
        2024: (
            (DEC, 23, DEC, 14),
            (DEC, 30, DEC, 28),
        ),
        2025: (
            (MAY, 2, MAY, 10),
            (NOV, 17, NOV, 8),
        ),
        2026: (
            (JAN, 2, JAN, 17),
            (JUN, 22, JUN, 27),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import os
import re
import unittest
from collections import Counter
from pathlib import Path
from unittest import mock

from polib import pofile as create_po_file

import holidays


class TestLocalization(unittest.TestCase):
    @mock.patch.dict(os.environ, {"LANGUAGE": "en_US"})
    def test_language_unavailable_en_us(self):
        self.assertEqual(os.environ["LANGUAGE"], "en_US")
        ca_xx = holidays.country_holidays("CA", language="xx")
        self.assertEqual(ca_xx["2022-01-01"], "New Year's Day")

    @mock.patch.dict(os.environ, {"LANGUAGE": "pl"})
    def test_language_unavailable_pl(self):
        self.assertEqual(os.environ["LANGUAGE"], "pl")
        pl_xx = holidays.country_holidays("PL", language="xx")
        self.assertEqual(pl_xx["2022-01-01"], "Nowy Rok")

    def test_localization(self):
        locale_dir = Path(__file__).parents[1] / "holidays" / "locale"
        placeholder_re = re.compile(r"%[a-zA-Z]")
        mandatory_fields = {
            "Project-Id-Version",
            "Report-Msgid-Bugs-To",
            "POT-Creation-Date",
            "PO-Revision-Date",
            "Last-Translator",
            "Language-Team",
            "Language",
            "MIME-Version",
            "Content-Type",
            "Content-Transfer-Encoding",
            "X-Source-Language",
        }

        for po_path in sorted(locale_dir.rglob("*.po")):
            try:
                po_file = create_po_file(po_path, check_for_duplicates=True)
            except ValueError as e:
                # Make sure no duplicated entries added.
                match = re.match(r"Entry (.*) already exists", str(e))
                self.assertEqual(
                    0,
                    len(match.groups()),
                    f"Entry `{match.group(1)}` already exists in {po_path}. "
                    "Please remove the duplicate.",
                )

                raise e

            missing_fields = mandatory_fields - set(po_file.metadata)
            self.assertFalse(
                missing_fields,
                f"{po_path} metadata does not contain mandatory fields: "
                f"{', '.join(sorted(missing_fields))}",
            )

            # Collect `<country_code>` part from
            # holidays/locale/<locale>/LC_MESSAGES/<country_code>.po.
            entity_code = po_path.stem
            # Collect `<locale>` part from
            # holidays/locale/<locale>/LC_MESSAGES/<country_code>.po.
            language = po_path.parts[-3]

            # Make sure no obsolete entries left.
            obsolete_entries = po_file.obsolete_entries()
            self.assertFalse(
                obsolete_entries,
                f"The {entity_code} {language} localization contains obsolete entries: "
                f"{', '.join(oe.msgid for oe in obsolete_entries)}",
            )

            entity = getattr(holidays, entity_code)

            # Skip original language files.
            if entity.default_language == language:
                continue

            # Make sure no entries left unlocalized.
            coverage = po_file.percent_translated()
            self.assertEqual(
                100,
                coverage,
                f"The {entity_code} {language} localization is incomplete ({coverage}% < 100%)",
            )

            for entry in po_file:
                self.assertEqual(
                    Counter(placeholder_re.findall(entry.msgid)),
                    Counter(placeholder_re.findall(entry.msgstr)),
                    f"The {entity_code} {language} localization contains placeholders "
                    f"mismatch in line {entry.linenum}: msgid `{entry.msgid}`, "
                    f"msgstr `{entry.msgstr}`.",
                )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.new_zealand import NewZealand
from tests.common import CommonCountryTests


class TestNZ(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1894, 2053)
        super().setUpClass(NewZealand)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2005-01-03",
            "2006-01-03",
            "2011-01-03",
            "2012-01-03",
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_day_after_new_years_day(self):
        name = "Day after New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-02" for year in self.full_range))
        obs_dts = (
            "2010-01-04",
            "2011-01-04",
            "2016-01-04",
            "2021-01-04",
            "2022-01-04",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_waitangi_day(self):
        name_1974 = "New Zealand Day"
        name_1977 = "Waitangi Day"

        self.assertHolidayName(name_1974, (f"{year}-02-06" for year in range(1974, 1977)))
        self.assertHolidayName(name_1977, (f"{year}-02-06" for year in range(1977, self.end_year)))
        self.assertNoHolidayName(
            name_1974, range(self.start_year, 1974), range(1977, self.end_year)
        )
        self.assertNoHolidayName(name_1977, range(self.start_year, 1977))

        self.assertSubdivNtlHolidayName(
            name_1977,
            "1964-02-03",
            "1965-02-08",
            "1966-02-07",
            "1967-02-06",
            "1968-02-05",
            "1969-02-03",
            "1970-02-09",
            "1971-02-08",
            "1972-02-07",
            "1973-02-05",
        )
        self.assertNoSubdivNtlHolidayName(
            name_1977, range(self.start_year, 1964), range(1974, 1977)
        )

        obs_dts = (
            "2016-02-08",
            "2021-02-08",
            "2022-02-07",
        )
        self.assertHolidayName(f"{name_1977} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anzac_day(self):
        name = "Anzac Day"
        self.assertHolidayName(name, (f"{year}-04-25" for year in range(1921, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1921))
        obs_dts = (
            "2015-04-27",
            "2020-04-27",
            "2021-04-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_sovereigns_birthday(self):
        name_king = "King's Birthday"
        name_queen = "Queen's Birthday"

        self.assertHolidayName(
            name_king,
            (f"{year}-11-09" for year in range(1902, 1912)),
            (f"{year}-06-03" for year in range(1912, 1936)),
            "1936-06-23",
            "1937-06-09",
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertHolidayName(name_king, range(1936, 1952), range(2023, self.end_year))
        self.assertNoHolidayName(name_king, range(self.start_year, 1902), range(1952, 2023))

        self.assertHolidayName(
            name_queen,
            "1952-06-02",
            "2017-06-05",
            "2018-06-04",
            "2019-06-03",
            "2020-06-01",
            "2021-06-07",
            "2022-06-06",
        )
        self.assertHolidayName(name_queen, range(1952, 2023))
        self.assertNoHolidayName(
            name_queen, range(self.start_year, 1952), range(2023, self.end_year)
        )

    def test_matariki(self):
        name = "Matariki"
        self.assertHolidayName(
            name,
            "2022-06-24",
            "2023-07-14",
            "2024-06-28",
            "2025-06-20",
            "2026-07-10",
            "2027-06-25",
            "2028-07-14",
            "2029-07-06",
            "2030-06-21",
            "2031-07-11",
            "2032-07-02",
            "2033-06-24",
            "2034-07-07",
            "2035-06-29",
            "2036-07-18",
            "2037-07-10",
            "2038-06-25",
            "2039-07-15",
            "2040-07-06",
            "2041-07-19",
            "2042-07-11",
            "2043-07-03",
            "2044-06-24",
            "2045-07-07",
            "2046-06-29",
            "2047-07-19",
            "2048-07-03",
            "2049-06-25",
            "2050-07-15",
            "2051-06-30",
            "2052-06-21",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2022))

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(
            name,
            "1907-10-09",
            "1908-10-14",
            "1909-10-13",
            "2020-10-26",
            "2021-10-25",
            "2022-10-24",
            "2023-10-23",
            "2024-10-28",
            "2025-10-27",
        )
        self.assertHolidayName(name, range(1900, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1900))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-28",
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_auckland_anniversary_day(self):
        name = "Auckland Anniversary Day"
        self.assertNoHolidayName(name)
        dts = (
            "2020-01-27",
            "2021-02-01",
            "2022-01-31",
            "2023-01-30",
            "2024-01-29",
            "2025-01-27",
        )
        self.assertSubdivAukHolidayName(name, dts)
        self.assertSubdivAukHolidayName(name, self.full_range)
        self.assertSubdivNtlHolidayName(name, dts)
        self.assertSubdivNtlHolidayName(
            name, range(self.start_year, 1964), range(1974, self.end_year)
        )

    def test_canterbury_anniversary_day(self):
        name = "Canterbury Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivCanHolidayName(
            name,
            "2020-11-13",
            "2021-11-12",
            "2022-11-11",
            "2023-11-17",
            "2024-11-15",
            "2025-11-14",
        )
        self.assertSubdivCanHolidayName(name, self.full_range)

    def test_chatham_islands_anniversary_day(self):
        name = "Chatham Islands Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivCitHolidayName(
            name,
            "2020-11-30",
            "2021-11-29",
            "2022-11-28",
            "2023-11-27",
            "2024-12-02",
            "2025-12-01",
        )
        self.assertSubdivCitHolidayName(name, self.full_range)

    def test_hawkes_bay_anniversary_day(self):
        name = "Hawke's Bay Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivHkbHolidayName(
            name,
            "2020-10-23",
            "2021-10-22",
            "2022-10-21",
            "2023-10-20",
            "2024-10-25",
            "2025-10-24",
        )
        self.assertSubdivHkbHolidayName(name, self.full_range)

    def test_marlborough_anniversary_day(self):
        name = "Marlborough Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivMbhHolidayName(
            name,
            "2020-11-02",
            "2021-11-01",
            "2022-10-31",
            "2023-10-30",
            "2024-11-04",
            "2025-11-03",
        )
        self.assertSubdivMbhHolidayName(name, self.full_range)

    def test_nelson_anniversary_day(self):
        name = "Nelson Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivNsnHolidayName(
            name,
            "2020-02-03",
            "2021-02-01",
            "2022-01-31",
            "2023-01-30",
            "2024-01-29",
            "2025-02-03",
        )
        self.assertSubdivNsnHolidayName(name, self.full_range)

    def test_otago_anniversary_day(self):
        name = "Otago Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivOtaHolidayName(
            name,
            "2020-03-23",
            "2021-03-22",
            "2022-03-21",
            "2023-03-20",
            "2024-03-25",
            "2025-03-24",
        )
        self.assertSubdivOtaHolidayName(name, self.full_range)

    def test_south_canterbury_anniversary_day(self):
        name = "South Canterbury Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivSouthCanterburyHolidayName(
            name,
            "2020-09-28",
            "2021-09-27",
            "2022-09-26",
            "2023-09-25",
            "2024-09-23",
            "2025-09-22",
        )
        self.assertSubdivSouthCanterburyHolidayName(name, self.full_range)

    def test_southland_anniversary_day(self):
        name = "Southland Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivStlHolidayName(
            name,
            "2009-01-19",
            "2010-01-18",
            "2011-01-17",
            "2020-04-14",
            "2021-04-06",
            "2022-04-19",
            "2023-04-11",
            "2024-04-02",
            "2025-04-22",
        )
        self.assertSubdivStlHolidayName(name, self.full_range)

    def test_taranaki_anniversary_day(self):
        name = "Taranaki Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivTkiHolidayName(
            name,
            "2020-03-09",
            "2021-03-08",
            "2022-03-14",
            "2023-03-13",
            "2024-03-11",
            "2025-03-10",
        )
        self.assertSubdivTkiHolidayName(name, self.full_range)

    def test_wellington_anniversary_day(self):
        name = "Wellington Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivWgnHolidayName(
            name,
            "2020-01-20",
            "2021-01-25",
            "2022-01-24",
            "2023-01-23",
            "2024-01-22",
            "2025-01-20",
        )
        self.assertSubdivWgnHolidayName(name, self.full_range)

    def test_west_coast_anniversary_day(self):
        name = "West Coast Anniversary Day"
        self.assertNoHolidayName(name)
        self.assertSubdivWtcHolidayName(
            name,
            "2020-11-30",
            "2021-11-29",
            "2022-11-28",
            "2023-12-04",
            "2024-12-02",
            "2025-12-01",
        )
        self.assertSubdivWtcHolidayName(name, self.full_range)

    def test_all_holidays_present(self):
        all_subdivisions = set(NewZealand.subdivisions)
        holidays_1969 = set()
        for subdiv in all_subdivisions:
            holidays_1969.update(NewZealand(years=1969, subdiv=subdiv, observed=False).values())
        holidays_2015 = set()
        for subdiv in all_subdivisions:
            holidays_2015.update(NewZealand(years=2015, subdiv=subdiv, observed=False).values())
        holidays_1974 = set()
        for subdiv in all_subdivisions:
            holidays_1974.update(NewZealand(years=1974, subdiv=subdiv, observed=False).values())

        all_holidays = {
            "New Year's Day",
            "Day after New Year's Day",
            "Waitangi Day",
            "Good Friday",
            "Easter Monday",
            "Anzac Day",
            "Queen's Birthday",
            "Labour Day",
            "Christmas Day",
            "Boxing Day",
            "Auckland Anniversary Day",
            "Taranaki Anniversary Day",
            "Hawke's Bay Anniversary Day",
            "Wellington Anniversary Day",
            "Marlborough Anniversary Day",
            "Nelson Anniversary Day",
            "Canterbury Anniversary Day",
            "South Canterbury Anniversary Day",
            "West Coast Anniversary Day",
            "Otago Anniversary Day",
            "Southland Anniversary Day",
            "Chatham Islands Anniversary Day",
        }
        self.assertEqual(all_holidays, holidays_1969)
        self.assertEqual(all_holidays, holidays_2015)
        all_holidays.remove("Waitangi Day")
        all_holidays.add("New Zealand Day")
        self.assertEqual(all_holidays, holidays_1974)

    def test_deprecated(self):
        # Deprecated Code, Remapped Code.
        for subdiv1, subdiv2 in (
            ("New Plymouth", "TKI"),
            ("STC", "South Canterbury"),
            ("Westland", "WTC"),
            ("WTL", "WTC"),
        ):
            self.assertEqual(
                NewZealand(subdiv=subdiv1, years=2022).keys(),
                NewZealand(subdiv=subdiv2, years=2022).keys(),
            )

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.brasil_bolsa_balcao import BrasilBolsaBalcao
from tests.common import CommonFinancialTests


class TestBrasilBolsaBalcao(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(BrasilBolsaBalcao)

    def test_universal_fraternization_day(self):
        name = "Confraternização Universal"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

    def test_carnival(self):
        name = "Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
            "2024-02-12",
            "2024-02-13",
        )
        self.assertHolidayNameCount(name, 2, self.full_range)

    def test_holy_thursday(self):
        name = "Quinta-feira Santa"
        self.assertHolidayName(
            name,
            "1995-04-13",
            "1996-04-04",
            "1997-03-27",
            "1998-04-09",
            "1999-04-01",
        )
        self.assertHolidayName(name, range(self.start_year, 2000))
        self.assertNoHolidayName(name, range(2000, self.end_year))

    def test_good_friday(self):
        name = "Sexta-feira Santa"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_tiradentes_day(self):
        name = "Tiradentes"
        years_absent = {1931, 1932}
        self.assertHolidayName(
            name, (f"{year}-04-21" for year in self.full_range if year not in years_absent)
        )
        self.assertNoHolidayName(name, years_absent)

    def test_workers_day(self):
        name = "Dia do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1925, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1925))

    def test_corpus_christi_day(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
        )
        self.assertHolidayName(name, self.full_range)

    def test_independence_day(self):
        name = "Independência do Brasil"
        self.assertHolidayName(name, (f"{year}-09-07" for year in self.full_range))

    def test_our_lady_of_aparecida(self):
        name = "Nossa Senhora Aparecida"
        self.assertHolidayName(name, (f"{year}-10-12" for year in range(1980, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1980))

    def test_all_souls_day(self):
        name = "Finados"
        self.assertHolidayName(name, (f"{year}-11-02" for year in self.full_range))

    def test_republic_proclamation_day(self):
        name = "Proclamação da República"
        self.assertHolidayName(name, (f"{year}-11-15" for year in self.full_range))

    def test_national_day_of_zumbi_and_black_awareness(self):
        name = "Dia Nacional de Zumbi e da Consciência Negra"
        self.assertHolidayName(name, (f"{year}-11-20" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_christmas_day(self):
        name = "Natal"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1922, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1922))

    def test_2022(self):
        self.assertHolidays(
            BrasilBolsaBalcao(years=2022),
            ("2022-01-01", "Confraternização Universal"),
            ("2022-02-28", "Carnaval"),
            ("2022-03-01", "Carnaval"),
            ("2022-04-15", "Sexta-feira Santa"),
            ("2022-04-21", "Tiradentes"),
            ("2022-05-01", "Dia do Trabalhador"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-09-07", "Independência do Brasil"),
            ("2022-10-12", "Nossa Senhora Aparecida"),
            ("2022-11-02", "Finados"),
            ("2022-11-15", "Proclamação da República"),
            ("2022-12-25", "Natal"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Confraternização Universal"),
            ("2023-02-20", "Carnaval"),
            ("2023-02-21", "Carnaval"),
            ("2023-04-07", "Sexta-feira Santa"),
            ("2023-04-21", "Tiradentes"),
            ("2023-05-01", "Dia do Trabalhador"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-09-07", "Independência do Brasil"),
            ("2023-10-12", "Nossa Senhora Aparecida"),
            ("2023-11-02", "Finados"),
            ("2023-11-15", "Proclamação da República"),
            ("2023-12-25", "Natal"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "Universal Fraternization Day"),
            ("2023-02-20", "Carnival"),
            ("2023-02-21", "Carnival"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-21", "Tiradentes' Day"),
            ("2023-05-01", "Worker's Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-09-07", "Independence Day"),
            ("2023-10-12", "Our Lady of Aparecida"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-15", "Republic Proclamation Day"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "День всесвітнього братання"),
            ("2023-02-20", "Карнавал"),
            ("2023-02-21", "Карнавал"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-21", "День Тирадентіса"),
            ("2023-05-01", "День трудящих"),
            ("2023-06-08", "Свято Тіла і Крові Христових"),
            ("2023-09-07", "День незалежності Бразилії"),
            ("2023-10-12", "День Богоматері Апаресіди"),
            ("2023-11-02", "День усіх померлих"),
            ("2023-11-15", "День проголошення республіки"),
            ("2023-12-25", "Різдво Христове"),
        )
